﻿using Grayscale.P007_SfenReport.L00025_Report;
using Grayscale.P007_SfenReport.L050_Report;
using Grayscale.P025_KifuLarabe.L00012_Atom;
using Grayscale.P025_KifuLarabe.L00025_Struct;
using Grayscale.P025_KifuLarabe.L00050_StructShogi;
using Grayscale.P025_KifuLarabe.L002_GraphicLog;
using Grayscale.P025_KifuLarabe.L004_StructShogi;
using Grayscale.P025_KifuLarabe.L007_Random;
using Grayscale.P025_KifuLarabe.L012_Common;
using Grayscale.P025_KifuLarabe.L100_KifuIO;
using Grayscale.P027_SasiteSeisei.L100_Sasiteseisei_FukasaYusen;
using Grayscale.P040_Kokoro.L00050_Kokoro;
using Grayscale.P043_Atama_Unou.L100_Unou;
using Grayscale.P045_Atama.L00025_KyHandan;
using Grayscale.P050_KifuWarabe.L00025_UsiLoop;
using Grayscale.P050_KifuWarabe.L00049_Kokoro;
using Grayscale.P050_KifuWarabe.L00050_KyHyoka;
using Grayscale.P050_KifuWarabe.L00051_Minimax;
using Grayscale.P050_KifuWarabe.L00052_Shogisasi;
using Grayscale.P050_KifuWarabe.L003_Kokoro;
using Grayscale.P050_KifuWarabe.L009_KyHyoka;
using Grayscale.P050_KifuWarabe.L010_Minimax;
using Grayscale.P050_KifuWarabe.L012_ScoreSibori;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using Grayscale.P027_SasiteSeisei.L080_Sasiteseisei;
using Grayscale.P027_SasiteSeisei.L00030_Sasiteseisei;
using Grayscale.P025_KifuLarabe.L003_Struct;

namespace Grayscale.P050_KifuWarabe.L030_Shogisasi
{
    /// <summary>
    /// 将棋指しエンジン。
    /// 
    /// 指し手を決めるエンジンです。
    /// </summary>
    public class ShogisasiImpl : Shogisasi
    {
        private ShogiEngine Owner { get { return this.owner; } }
        private ShogiEngine owner;

        public static ReportEnvironment REPORT_ENVIRONMENT;
        static ShogisasiImpl()
        {
            ShogisasiImpl.REPORT_ENVIRONMENT = new ReportEnvironmentImpl(
                        "../../Logs/局面画像ﾛｸﾞ/",//argsDic["outFolder"],
                        "../../Data/img/gkLog/",//argsDic["imgFolder"],
                        "koma1.png",//argsDic["kmFile"],
                        "suji1.png",//argsDic["sjFile"],
                        "20",//argsDic["kmW"],
                        "20",//argsDic["kmH"],
                        "8",//argsDic["sjW"],
                        "12"//argsDic["sjH"]
                        );
        }

        /// <summary>
        /// 心エンジン。
        /// </summary>
        public Kokoro Kokoro { get; set; }

        /// <summary>
        /// 点数付けエンジン。
        /// </summary>
        public MinimaxEngine MinimaxEngine { get; set; }

        /// <summary>
        /// 枝狩りエンジン。
        /// </summary>
        public ScoreSiboriEngine EdagariEngine { get; set; }

        /// <summary>
        /// 思考エンジンの性格。
        /// </summary>
        public Seikaku Seikaku { get; set; }

        private KyHyokaWriter kyHyokaWriter;

        /// <summary>
        /// 右脳。
        /// </summary>
        private Unou Unou { get; set; }

        public ShogisasiImpl(ShogiEngine owner)
        {
            this.owner = owner;
            this.Kokoro = new KokoroImpl(this.owner);
            this.MinimaxEngine = new MinimaxEngineImpl(this.owner);
            this.EdagariEngine = new ScoreSiboriEngine();

            this.Seikaku = new Seikaku();
            this.kyHyokaWriter = new KyHyokaWriterImpl();

            this.Unou = new Unou();
            this.Unou.Make_Random();
        }

        /// <summary>
        /// 対局開始のとき。
        /// </summary>
        public void OnTaikyokuKaisi()
        {
            this.Kokoro.ClearTenonagare();

            // 作った妄想は履歴に追加。

            // 重要さ
            double ordKkp = 10000.0d;//三駒関係ＫＫＰ  ※26.333とか返ってくるので、10000倍して 260,000 ぐらいにします。
            double ordKmdk = 1000.0d;//駒得
            double ordHmdk = 510.0d;//紐付き
            double ordKmgr = 170.0d;//きまぐれ（紐付きの1/3ぐらいで）
            double ordGm = 100.0d;//玉の守り
            double ord7 =  50.0d;
            double ord8 =  45.0d;
            double ordKahd = 10.0d;//角頭の紐付き（うまく働いていないので下げる）


            //--------------------------------------------------------------------------------------------------------------
            //
            // テスト中。
            //

            // 「三駒関係ＫＫＰ」タイプの狙いを作ります。
            this.Kokoro.AddTenonagare(new TenonagareImpl(null, TenonagareName.SankomaKankeiKkp, ordKkp, null, null, new Basho(0), OwataMinister.WARABE_ENGINE));

            //--------------------------------------------------------------------------------------------------------------
            //
            // 優先しても悪さしないもの。
            //

            // 「駒得」タイプの狙いを作ります。
            this.Kokoro.AddTenonagare(new TenonagareImpl(null, TenonagareName.KomaDoku, ordKmdk, null, null, new Basho(0), OwataMinister.WARABE_ENGINE));

            //--------------------------------------------------------------------------------------------------------------
            //
            // 強さの頼みの綱。
            //

            // 「紐付き」タイプの狙いを作ります。
            this.Kokoro.AddTenonagare(new TenonagareImpl(null, TenonagareName.Himoduki, ordHmdk, null, null, new Basho(0), OwataMinister.WARABE_ENGINE));

            //--------------------------------------------------------------------------------------------------------------
            //
            // 面白くするための苦肉の策。
            //

            // 「気まぐれ」タイプの狙いを作ります。
            this.Kokoro.AddTenonagare(new TenonagareImpl(null, TenonagareName.Kimagure, ordKmgr, null, null, new Basho(0), OwataMinister.WARABE_ENGINE));

            //--------------------------------------------------------------------------------------------------------------

            // 「玉の守り」タイプの狙いを作ります。
            this.Kokoro.AddTenonagare(new TenonagareImpl(null, TenonagareName.GyokuNoMamori, ordGm, null, null, new Basho(0), OwataMinister.WARABE_ENGINE));

            // 「飛車道が通っている」タイプの狙いを作ります。
            this.Kokoro.AddTenonagare(new TenonagareImpl(null, TenonagareName.Toosi, ord7, null, null, new Basho(0), OwataMinister.WARABE_ENGINE));

            // 「目の前の歩を取れ」タイプの狙いを作ります。
            this.Kokoro.AddTenonagare(new TenonagareImpl(null, TenonagareName.MenomaenoFuWoTore, ord8, null, null, new Basho(0), OwataMinister.WARABE_ENGINE));

            //--------------------------------------------------------------------------------------------------------------
            //
            // 故障部品
            //

            // 「角頭の紐付き」タイプの狙いを作ります。
            this.Kokoro.AddTenonagare(new TenonagareImpl(null, TenonagareName.KakuTouNoHimoduki, ordKahd, null, null, new Basho(0), OwataMinister.WARABE_ENGINE));

        }

        /// <summary>
        /// 指し手を決めます。
        /// </summary>
        /// <param name="enableLog"></param>
        /// <param name="isHonshogi"></param>
        /// <param name="kifu"></param>
        /// <param name="playerInfo"></param>
        /// <param name="logTag"></param>
        /// <returns></returns>
        public KifuNode WA_Bestmove(
            bool isHonshogi,
            KifuTree kifu,
            PlayerInfo playerInfo,
            KwErrorHandler owata
            )
        {

            try
            {
                // 「移動」タイプの狙いを、どんどん付け足します。
                this.Kokoro.Omoituki(
                    playerInfo.Playerside, (KifuNode)kifu.CurNode, this.Seikaku);
            }
            catch (Exception ex) { owata.DonimoNaranAkirameta(ex, "ベスト指し手のチョイスをしたときの前半１０です。"); throw ex; }

            try
            {
                //
                // 指し手生成ルーチンで、棋譜ツリーを作ります。
                //
                new Sasiteseisei_FukasaYusen_Routine().WAA_Yomu_Start(kifu, isHonshogi, owata);
            }
            catch (Exception ex) { owata.DonimoNaranAkirameta(ex, "棋譜ツリーを作っていたときです。"); throw ex; }



            // デバッグ用だが、メソッドはこのオブジェクトを必要としてしまう。
            GraphicalLog_File logF_kiki = new GraphicalLog_File();

            try
            {
                // 点数を付ける（葉ノードに点数を付け、途中のノードの点数も出します）
                this.MinimaxEngine.Tensuduke_ForeachLeafs(
                    Util_Sky.ToSfenSasiteText(kifu.CurNode.Key),
                    (KifuNode)kifu.CurNode,
                    kifu,
                    this.Kokoro,
                    this.Unou,
                    playerInfo,
                    ShogisasiImpl.REPORT_ENVIRONMENT,
                    logF_kiki,
                    owata
                );
            }
            catch (Exception ex) { owata.DonimoNaranAkirameta(ex, "ノードに点数を付けたときです。"); throw ex; }

#if DEBUG
            try
            {
                string sasiteText1 = Util_Sky.ToSfenSasiteText(kifu.CurNode.Key);
                KifuNode kifuNode1 = (KifuNode)kifu.CurNode;
                string relFolder = "" + Util_Sky.ToSfenSasiteText(kifu.CurNode.Key) + "/";

                // 評価明細のログ出力。
                this.kyHyokaWriter.Write_ForeachLeafs(
                    sasiteText1,
                    kifuNode1,
                    kifu,
                    playerInfo,
                    relFolder,
                    ShogisasiImpl.REPORT_ENVIRONMENT,
                    owata
                    );
            }
            catch (Exception ex)
            {
                //>>>>> エラーが起こりました。
                string message = ex.GetType().Name + " " + ex.Message + "：評価明細付きのログ出力をしていたときです。：";
                Debug.Fail(message);

                // どうにもできないので  ログだけ取って、上に投げます。
                OwataMinister.WARABE_ENGINE.LogTag.WriteLine_Error(message);
                throw ex;
            }
#endif
            try
            {
#if DEBUG
                if (0 < logF_kiki.boards.Count)//ﾛｸﾞが残っているなら
                {
                    //
                    // ログの書き出し
                    //
                    Util_GraphicalLog.Log(
                        true,//enableLog,
                        "#評価ログ",
                        "[" + Util_GraphicalLog.BoardFileLog_ToJsonStr(logF_kiki) + "]"
                    );

                    // 書き出した分はクリアーします。
                    logF_kiki.boards.Clear();
                }
#endif

                // 評価値の高いノードだけを残します。
                this.EdagariEngine.EdaSibori_HighScore(kifu, this.Kokoro);
            }
            catch (Exception ex) { owata.DonimoNaranAkirameta(ex, "ベスト指し手のチョイスをしたときの後半２０です。"); throw ex; }


            // 評価値の同点があれば、同点決勝をして　1手に決めます。
            KifuNode bestKifuNode = null;
            try
            {
                bestKifuNode = this.ChoiceNode_DoutenKessyou(kifu, isHonshogi, owata);
            }
            catch (Exception ex) { owata.DonimoNaranAkirameta(ex, "ベスト指し手のチョイスをしたときの後半３０です。"); throw ex; }


            return bestKifuNode;
        }


        /// <summary>
        /// 同点決勝。
        /// 
        /// 評価値が同点のノード（指し手）の中で、ランダムに１つ選びます。
        /// </summary>
        /// <param name="kifu">ツリー構造になっている棋譜</param>
        /// <param name="logTag">ログ</param>
        /// <returns></returns>
        private KifuNode ChoiceNode_DoutenKessyou(
            KifuTree kifu,
            bool isHonshogi, KwErrorHandler owata)
        {
            KifuNode bestKifuNode = null;

            {
                // 次のノードをリストにします。
                List<KifuNode> nextNodes = Converter04.NextNodes_ToList(kifu.CurNode);

                // 次のノードをシャッフル済みリストにします。
                List<KifuNode> nextNodes_shuffled = Converter04.NextNodes_ToList(kifu.CurNode);
                LarabeShuffle<KifuNode>.Shuffle_FisherYates(ref nextNodes_shuffled);

                // シャッフルした最初のノードを選びます。
                if (0 < nextNodes_shuffled.Count)
                {
                    bestKifuNode = nextNodes_shuffled[0];
                }
            }

            return bestKifuNode;
        }



    }
}
